@@ -1,14 +1,16 @@
+import warnings
 from dataclasses import dataclass
 from types import new_class
 from typing import Callable, Optional, TYPE_CHECKING, Tuple, Type, Union
 
 from apischema.conversions import Conversion
+from apischema.conversions.conversions import ResolvedConversion
 from apischema.conversions.utils import identity
 from apischema.dataclasses import replace
-from apischema.utils import PREFIX, cached_property
+from apischema.utils import PREFIX
 
 if TYPE_CHECKING:
-    from apischema.deserialization.coercion import Coercion
+    from apischema.deserialization.coercion import Coerce
 
 Model = Union[Type, Callable[[], Type]]
 
@@ -22,6 +24,8 @@ def check_model(origin: Type, model: Type):
 
 MODEL_ORIGIN_ATTR = f"{PREFIX}model_origin"
 
+DATACLASS_ATTR = "_dataclass"
+
 
 @dataclass(frozen=True)
 class DataclassModel:
@@ -29,21 +33,24 @@ class DataclassModel:
     model: Model
     fields_only: bool
 
-    @cached_property
+    @property
     def dataclass(self) -> Type:
-        origin = self.origin
-        if isinstance(self.model, type):
-            assert check_model(origin, self.model) is None
-            model = self.model
-        else:
-            model = self.model()
-            check_model(origin, model)
-        namespace = {"__new__": lambda _, *args, **kwargs: origin(*args, **kwargs)}
-        if not self.fields_only:
-            namespace[MODEL_ORIGIN_ATTR] = origin
-        return new_class(
-            model.__name__, (model,), exec_body=lambda ns: ns.update(namespace)
-        )
+        if not hasattr(self, "_dataclass"):
+            origin = self.origin
+            if isinstance(self.model, type):
+                assert check_model(origin, self.model) is None
+                model = self.model
+            else:
+                model = self.model()
+                check_model(origin, model)
+            namespace = {"__new__": lambda _, *args, **kwargs: origin(*args, **kwargs)}
+            if not self.fields_only:
+                namespace[MODEL_ORIGIN_ATTR] = origin
+            cls = new_class(
+                model.__name__, (model,), exec_body=lambda ns: ns.update(namespace)
+            )
+            object.__setattr__(self, "_dataclass", cls)
+        return getattr(self, "_dataclass")
 
 
 def dataclass_model(
@@ -52,10 +59,14 @@ def dataclass_model(
     *,
     fields_only: bool = False,
     additional_properties: Optional[bool] = None,
-    coercion: Optional["Coercion"] = None,
-    default_fallback: Optional[bool] = None,
+    coercion: Optional["Coerce"] = None,
+    fall_back_on_default: Optional[bool] = None,
     exclude_unset: Optional[bool] = None,
 ) -> Tuple[Conversion, Conversion]:
+    warnings.warn(
+        "dataclass_model is deprecated, use set_object_fields instead",
+        DeprecationWarning,
+    )
     if isinstance(model, type):
         check_model(origin, model)
 
@@ -63,8 +74,8 @@ def dataclass_model(
     conversion = Conversion(
         identity,
         additional_properties=additional_properties,
-        coercion=coercion,
-        default_fallback=default_fallback,
+        coerce=coercion,
+        fall_back_on_default=fall_back_on_default,
         exclude_unset=exclude_unset,
     )
     d_conv = replace(conversion, source=model_type, target=origin)
@@ -78,3 +89,12 @@ def has_model_origin(cls: Type) -> bool:
 
 def get_model_origin(cls: Type) -> Type:
     return getattr(cls, MODEL_ORIGIN_ATTR)
+
+
+def handle_dataclass_model(conversion: ResolvedConversion) -> ResolvedConversion:
+    conv: Conversion = conversion
+    if isinstance(conv.source, DataclassModel):
+        conv = replace(conv, source=conv.source.dataclass)
+    if isinstance(conv.target, DataclassModel):
+        conv = replace(conv, target=conv.target.dataclass)
+    return ResolvedConversion(conv)
