@@ -1,13 +1,16 @@
 from typing import Pattern
 
+from apischema.conversions.conversions import DefaultConversion
 from apischema.types import AnyType
 
 
-def infer_pattern(tp: AnyType) -> Pattern:
-    from apischema.json_schema.generation.schema import DeserializationSchemaBuilder
+def infer_pattern(tp: AnyType, default_conversion: DefaultConversion) -> Pattern:
+    from apischema.json_schema.schema import DeserializationSchemaBuilder
 
     try:
-        builder = DeserializationSchemaBuilder(lambda s: s, lambda s: s, {}, False)
+        builder = DeserializationSchemaBuilder(
+            False, lambda s: s, default_conversion, False, lambda r: r, {}
+        )
         prop_schema = builder.visit(tp)
     except RecursionError:
         pass
