@@ -20,11 +20,11 @@ from pathlib import (
     PureWindowsPath,
     WindowsPath,
 )
-from typing import Deque, List, NewType, TypeVar
+from typing import Deque, List, TypeVar
 from uuid import UUID
 
-from apischema.conversions import Conversion, as_str, deserializer, serializer
-from apischema.json_schema.schema import schema
+from apischema import deserializer, schema, serializer, type_name
+from apischema.conversions import Conversion, as_str
 
 T = TypeVar("T")
 
@@ -39,6 +39,10 @@ def to_base64(b: bytes) -> str:
     return b64encode(b).decode()
 
 
+type_name(graphql="Bytes")(bytes)
+schema(encoding="base64")(bytes)
+
+
 # ================ collections ==================
 
 deserializer(Conversion(deque, source=List[T], target=Deque[T]))
@@ -52,34 +56,44 @@ if sys.version_info < (3, 7):
 
 if sys.version_info >= (3, 7):  # pragma: no cover
     for cls, format in [(date, "date"), (datetime, "date-time"), (time, "time")]:
-        str_type = schema(format=format)(NewType(cls.__name__.capitalize(), str))
-        deserializer(Conversion(cls.fromisoformat, source=str_type, target=cls))  # type: ignore # noqa: E501
-        serializer(Conversion(cls.isoformat, source=cls, target=str_type))  # type: ignore # noqa: E501
+        deserializer(Conversion(cls.fromisoformat, source=str, target=cls))  # type: ignore
+        serializer(Conversion(cls.isoformat, source=cls, target=str))  # type: ignore
+        type_name(graphql=cls.__name__.capitalize())(cls)
+        schema(format=format)(cls)
 
 # ================== decimal ====================
 
 deserializer(Conversion(Decimal, source=float, target=Decimal))
 serializer(Conversion(float, source=Decimal, target=float))
+type_name(None)(Decimal)
 
 # ================= ipaddress ===================
 
-for cls in (IPv4Address, IPv4Interface, IPv4Network):
-    schema(format="ipv4")(as_str(cls))
-for cls in (IPv6Address, IPv6Interface, IPv6Network):
-    schema(format="ipv6")(as_str(cls))
+for classes, format in [
+    ((IPv4Address, IPv4Interface, IPv4Network), "ipv4"),
+    ((IPv6Address, IPv6Interface, IPv6Network), "ipv6"),
+]:
+    for cls in classes:
+        as_str(cls)
+        type_name(graphql=cls.__name__)(cls)
+        schema(format=format)(cls)
 
 # ==================== path =====================
 
 for cls in (PurePath, PurePosixPath, PureWindowsPath, Path, PosixPath, WindowsPath):
     as_str(cls)
+    type_name(None)(cls)
 
 # =================== pattern ===================
 
 Pattern = type(re.compile(r""))
 deserializer(Conversion(re.compile, source=str, target=Pattern))
 serializer(Conversion(lambda p: p.pattern, source=Pattern, target=str))
+type_name(None)(cls)
 
 
 # ==================== uuid =====================
 
-schema(format="uuid")(as_str(UUID))
+as_str(UUID)
+type_name(graphql="UUID")
+schema(format="uuid")(UUID)
